@@ -1,5 +1,6 @@
 # SPDX-License-Identifier: Apache-2.0
 
+from logging import getLogger
 import numpy as np
 from scipy.sparse import coo_matrix
 from ..proto import onnx_proto, TensorProto
@@ -10,6 +11,9 @@ from ..common._topology import Variable
 from ..common._registration import get_shape_calculator, get_converter
 
 
+logger = getLogger('skl2onnx')
+
+
 class GraphStateVar:
     pass
 
@@ -21,7 +25,8 @@ class GraphState:
                  options=None, expected_inputs=None,
                  expected_outputs=None, input_range=None,
                  output_range=None, operator=None,
-                 run_converters=False, **attrs):
+                 run_converters=False, input_types=None, **attrs):
+
         self.inputs = inputs
         self._output_names = output_names
         self._input_range = input_range.copy() if input_range else [1, 1e9]
@@ -49,6 +54,7 @@ class GraphState:
         self.onnx_prefix_name = onnx_prefix_name
         self.attrs = attrs
         self.options = options
+        self.input_types = input_types
 
         for att in ['inputs', '_expected_inputs',
                     '_expected_outputs', 'computed_inputs_',
@@ -287,15 +293,15 @@ class GraphState:
         if isinstance(output, tuple):
             if output[0] in output_names:
                 return output
-            return (scope.get_unique_variable_name(output[0]),
-                    output[1])
+            return (scope.get_unique_variable_name(output[0]), output[1])
         raise NotImplementedError(
             "Unexpected output type {} [{}]. "
             "You may raise an issue at https://github.com/onnx/"
             "sklearn-onnx/issues.".format(type(output), output))
 
     @staticmethod
-    def _update_inputs(inputs, names, scope, expected_inputs, input_range):
+    def _update_inputs(inputs, names, scope, expected_inputs,
+                       input_range, input_types=None):
         new_inputs = []
         for inp in inputs:
             if isinstance(inp, (Variable, tuple, GraphStateVar)):
@@ -312,9 +318,23 @@ class GraphState:
         for i in range(0, len(new_inputs)):
             inp = new_inputs[i]
             if isinstance(inp, tuple) and len(inp) == 2:
-                stype = None if isinstance(inp[1], str) else inp[1]
-                new_inputs[i] = Variable(
-                    inp[0], inp[0], type=stype, scope=scope)
+                if input_types is not None and i < len(input_types):
+                    stype = input_types[i]
+                else:
+                    stype = None if isinstance(inp[1], str) else inp[1]
+                if scope is not None:
+                    if inp[0] in scope.variables:
+                        var = scope.variables[inp[0]]
+                        if stype is not None:
+                            var.check_compatible_type(stype)
+                    else:
+                        onnx_name = scope.get_unique_variable_name(inp[0])
+                        var = Variable(
+                            inp[0], onnx_name, type=stype, scope=scope)
+                        scope.register_variable(var)
+                else:
+                    var = Variable(inp[0], inp[0], type=stype, scope=scope)
+                new_inputs[i] = var
                 inp = new_inputs[i]
             elif isinstance(inp, GraphStateVar):
                 new_inputs[i] = inp.as_variable(scope)
@@ -322,14 +342,16 @@ class GraphState:
             elif not isinstance(inp, Variable):
                 raise TypeError(
                     "Inputs %d - %r must be of type Variable." % (i, inp))
+
             if names is not None:
                 try:
-                    inp.onnx_name = (
+                    onnx_name = (
                         names[i] if isinstance(names[i], str)
                         else names[i][0])
                 except IndexError as e:
                     raise IndexError(
                         "Wrong index %d, list=%s." % (i, names)) from e
+                inp.set_onnx_name(onnx_name)
 
         # Second pass.
         if expected_inputs is not None:
@@ -349,10 +371,20 @@ class GraphState:
                                     j >= input_range[0]):
                                 continue
                             if new_inputs[j].type is not None:
-                                new_inputs[i].type = (
+                                new_inputs[i].set_type(
                                     new_inputs[j].type.__class__())
                                 break
 
+        # Overwrite types if input_types is specified.
+        if input_types is not None:
+            for i in range(len(new_inputs)):
+                if i >= len(input_types):
+                    raise RuntimeError(
+                        "Mismatch between computed inputs[%d]=%r and "
+                        "overwritten input_types[%d]=%r." % (
+                            i, new_inputs, i, input_types))
+                if input_types[i] is not None:
+                    new_inputs[i].type = input_types[i]
         return new_inputs
 
     @staticmethod
@@ -387,7 +419,7 @@ class GraphState:
             if hasattr(inp, 'type') and inp.type is None:
                 ct = expected1[i][1]
                 if ct in memo:
-                    vars1[i].type = copy_type(memo[ct][0])
+                    vars1[i].set_type(copy_type(memo[ct][0]))
             elif isinstance(inp, tuple):
                 ct = expected1[i][1]
                 if ct in memo:
@@ -417,7 +449,8 @@ class GraphState:
             self.computed_inputs_ = GraphState._update_inputs(
                 self.inputs, inputs, scope=self.scope,
                 expected_inputs=self._expected_inputs,
-                input_range=self._input_range)
+                input_range=self._input_range,
+                input_types=self.input_types)
 
             name = self.scope.get_unique_operator_name(self.onnx_prefix)
             if self.is_model:
@@ -427,13 +460,34 @@ class GraphState:
 
                 # a model is converted into a subgraph
                 sub_op_inputs = self.computed_inputs_
+                for v in sub_op_inputs:
+                    if not isinstance(v, Variable):
+                        raise TypeError(
+                            "Every input variable must be a Variable not %r,"
+                            " v=%r." % (type(v), v))
+                    scope = v.scope
+                    if hasattr(scope, 'variables'):
+                        if v.onnx_name not in scope.variables:
+                            raise RuntimeError(
+                                "Variable %r missing from scope "
+                                "(operator=%r, model=%r), list=%r." % (
+                                    v, self.operator,
+                                    type(self.operator_instance),
+                                    list(sorted(self.scope.variables))))
 
                 # output are not defined, we need to call a parser.
                 from .._parse import _parse_sklearn
                 self.scope.add_options(
                     id(self.operator_instance), self.options)
-                sub_outputs = _parse_sklearn(
-                    self.scope, self.operator_instance, sub_op_inputs)
+                try:
+                    sub_outputs = _parse_sklearn(
+                        self.scope, self.operator_instance, sub_op_inputs)
+                except RuntimeError as e:
+                    raise RuntimeError(
+                        "Unable to run parser for model type %r, inputs=%r "
+                        "(input_types=%r)." % (
+                            type(self.operator_instance), sub_op_inputs,
+                            self.input_types)) from e
                 set_input_names = set(v.onnx_name for v in sub_op_inputs)
                 sub_op = None
                 for op in self.scope.operators.values():
@@ -455,20 +509,31 @@ class GraphState:
                 sub_op.outputs = sub_outputs
 
                 shape_calc = get_shape_calculator(self.operator_name)
+                logger.debug("[StateShape] call %r fed %r - %r" % (
+                    sub_op,
+                    "".join(str(i.is_fed) for i in sub_op.inputs),
+                    "".join(str(i.is_fed) for i in sub_op.outputs)))
                 shape_calc(sub_op)
+                logger.debug("[StateShape] end - %r" % sub_op)
 
                 # Add Identity nodes to be consistent with `is_fed`
                 # in Topology.
-                if expected_outputs is not None:
+                if sub_op.outputs is not None and len(sub_op.outputs) > 0:
+                    outputs = [
+                        self.scope.declare_local_variable(
+                            o.onnx_name, type=o.type)
+                        for o in sub_op.outputs]
+                elif (expected_outputs is not None and
+                        len(expected_outputs) > 0):
                     outputs = [
                         self._get_output_name(
                             self._output_names, o, self.scope)
                         for o in expected_outputs]
                 else:
-                    outputs = [
-                        self.scope.declare_local_variable(
-                            o.onnx_name, type=o.type)
-                        for o in sub_op.outputs]
+                    raise RuntimeError(
+                        "sub_op.outputs is None as well as expected_outputs "
+                        "for operator %r." % sub_op)
+
                 if len(outputs) != len(sub_op.outputs):
                     raise RuntimeError(
                         "Mismatched number of outputs %s and %s." % (
@@ -482,14 +547,43 @@ class GraphState:
                 self.computed_outputs_ = outputs
                 self.computed_inputs2_ = sub_op.inputs
                 self.computed_outputs2_ = [
-                    (v.raw_name, v.type) for v in self.computed_outputs_]
+                    (v[0], v[1]) for v in self.computed_outputs_]
 
                 if self.run_converters:
                     # The parser was run on sub-operators but not the
                     # converter.
                     conv = get_converter(self.operator_name)
+                    logger.debug("[StateConv] %r fed %r - %r" % (
+                        sub_op,
+                        "".join(str(i.is_fed) for i in sub_op.inputs),
+                        "".join(str(i.is_fed) for i in sub_op.outputs)))
                     conv(self.scope, sub_op, self.container)
+                    logger.debug("[StateConv] %r - end." % sub_op)
+                else:
+                    if (expected_outputs is not None and
+                            len(sub_op.outputs) == len(expected_outputs)):
+                        for v1, v2 in zip(sub_op.outputs, expected_outputs):
+                            if isinstance(v2, tuple):
+                                v2 = v2[0]
+                            if (hasattr(v1, 'onnx_name') and
+                                    hasattr(v2, 'onnx_name')):
+                                if v1.onnx_name != v2.onnx_name:
+                                    # One identity is missing
+                                    n = self.scope.get_unique_operator_name(
+                                        'idgstate')
+                                    self.container.add_node(
+                                        'Identity', [v1.onnx_name],
+                                        [v2.onnx_name], name=n)
             else:
+
+                def _name_(obj):
+                    if isinstance(obj, tuple) and len(obj) == 2:
+                        return obj[0]
+                    if hasattr(obj, 'onnx_name'):
+                        return obj.onnx_name
+                    raise TypeError(
+                        "Unable to extract variable name from %r." % obj)
+
                 # only one node is added
                 if self.options is not None:
                     raise RuntimeError(
@@ -498,8 +592,8 @@ class GraphState:
                 outputs = [
                     self._get_output_name(self._output_names, o, self.scope)
                     for o in expected_outputs]
-                input_names = [i[0] for i in inputs]
-                output_names = [i[0] for i in outputs]
+                input_names = [_name_(i) for i in inputs]
+                output_names = [_name_(i) for i in outputs]
                 self.container.add_node(
                     self.operator_name, input_names, output_names,
                     name=name, **self.attrs)
@@ -517,9 +611,10 @@ class GraphState:
                     if isinstance(kind, str):
                         self.computed_outputs_.append((name, kind))
                     else:
-                        var = self.scope.declare_local_variable(name, kind)
+                        var = self.scope.declare_local_variable(
+                            name, kind, missing_type=True)
                         # name already comes from
                         # scope.get_unique_variable_name
-                        var.onnx_name = name
-                        var.is_fed = True
+                        var.set_onnx_name(name)
+                        var.init_status(is_fed=True)
                         self.computed_outputs_.append(var)
