@@ -6,6 +6,7 @@ import re
 import sys
 import traceback
 import warnings
+from logging import getLogger
 import numpy as np
 from scipy.sparse import coo_matrix
 from onnx.defs import onnx_opset_version, get_all_schemas_with_history
@@ -22,10 +23,12 @@ except ImportError:
     # onnx is too old.
     SparseTensorProto = None
     make_sparse_tensor = None
-from .interface import ModelContainer
 from .utils import get_domain
 
 
+logger = getLogger('skl2onnx')
+
+
 def _get_operation_list():
     """
     Investigates this module to extract all ONNX functions
@@ -197,7 +200,7 @@ class SklearnModelContainerNode(RawModelContainerNode):
             self._outputs.append(variable)
 
 
-class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
+class ModelComponentContainer(_WhiteBlackContainer):
     """
     In the conversion phase, this class is used to collect all materials
     required to build an *ONNX* *GraphProto*, which is encapsulated in a
@@ -249,6 +252,79 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
         # All registered models.
         self.registered_models = registered_models
 
+    def swap_names(self, old_name, new_name):
+        """
+        Swaps variables names.
+
+        :param old_name: old name
+        :param new_name: new name
+        :return: list of impacted objects
+        """
+        exc_list = {'Scan', 'Loop', 'If'}
+        for node in self.nodes:
+            if node.op_type not in exc_list:
+                continue
+            if (old_name in node.input or old_name in node.output or
+                    new_name in node.input or new_name in node.output):
+                raise NotImplementedError(
+                    "Unable to handle subgraphs for node type %r."
+                    "(%r, %r)" % (node.op_type, old_name, new_name))
+        res = []
+
+        for inp in self.inputs:
+            if inp.name == old_name:
+                inp.name = new_name
+                res.append(('Io', inp))
+            elif inp.name == new_name:
+                inp.name = old_name
+                res.append(('In', inp))
+
+        for inp in self.outputs:
+            if inp.name == old_name:
+                inp.name = new_name
+                res.append(('Oo', inp))
+            elif inp.name == new_name:
+                inp.name = old_name
+                res.append(('On', inp))
+
+        for inp in self.initializers:
+            if inp.name == old_name:
+                inp.name = new_name
+                res.append(('-o', inp))
+            elif inp.name == new_name:
+                inp.name = old_name
+                res.append(('-n', inp))
+
+        for node in self.nodes:
+            modified = False
+            new_input = []
+            for name in node.input:
+                if name == old_name:
+                    name = new_name
+                    modified = True
+                elif name == new_name:
+                    name = old_name
+                    modified = True
+                new_input.append(name)
+            new_output = []
+            for name in node.output:
+                if name == old_name:
+                    name = new_name
+                    modified = True
+                elif name == new_name:
+                    name = old_name
+                    modified = True
+                new_output.append(name)
+            if modified:
+                if node.op_type in exc_list:
+                    raise NotImplementedError(
+                        "Unable to handle subgraphs for node type %r."
+                        "" % node.op_type)
+                node.input[:] = new_input[:]
+                node.output[:] = new_output[:]
+                res.append(("n-", node))
+        return res
+
     def __str__(self):
         """
         Shows internal information.
@@ -340,6 +416,7 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
                         or a float array).
         :return: created tensor
         """
+        logger.debug("[Init] %r, %r, %r" % (name, onnx_type, shape))
         sparse_tensor = None
         tensor = None
 
@@ -504,6 +581,8 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
             inputs = [inputs]
         if isinstance(outputs, str):
             outputs = [outputs]
+        logger.debug("[Node] %r - %r -> %r (name=%r)" % (
+            op_type, ",".join(inputs), ",".join(outputs), name))
         try:
             common = set(inputs) & set(outputs)
         except TypeError as e:
@@ -566,9 +645,11 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
                 op_version > self.target_opset_any_domain(op_domain)):
             raise RuntimeError(
                 "Opset number {} is higher than targeted opsets {} for "
-                "node '{}' (domain: '{}').".format(
+                "node type '{}' name='{}' input={} "
+                "output={} (domain='{}').".format(
                     op_version, self.target_opset_all,
-                    node.op_type, op_domain))
+                    node.op_type, node.name,
+                    node.input, node.output, op_domain))
 
     def target_opset_any_domain(self, domain):
         target_opset = self.target_opset_all
@@ -703,7 +784,9 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
         for inp in self.initializers:
             name = inp.name
             order[name] = 0
+
         n_iter = 0
+        missing_ops = []
         while n_iter < len(self.nodes) * 2:
             n_iter += 1
             missing_names = set()
@@ -730,8 +813,8 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
                     if name in order:
                         raise RuntimeError(
                             "Unable to sort a node (cycle). An output was "
-                            "already ordered %r (iteration=%r)." % (
-                                name, n_iter))
+                            "already ordered with name %r (iteration=%r)."
+                            "" % (name, n_iter))
                     order[name] = maxi
             if len(missing_names) == 0:
                 continue
@@ -750,8 +833,8 @@ class ModelComponentContainer(ModelContainer, _WhiteBlackContainer):
             rows.append("--")
             rows.append("--all-nodes--")
             rows.append("--")
-            rows.extend("%s(%s) -> [%s]" % (
-                n.name or n.op_type,
+            rows.extend("%s|%s(%s) -> [%s]" % (
+                n.op_type, n.name or n.op_type,
                 ', '.join(map(nstr, n.input)),
                 ', '.join(n.output))
                 for n in self.nodes)
