@@ -70,6 +70,28 @@ def fit_classification_model(model, n_classes, is_int=False,
     return model, X_test
 
 
+def fit_clustering_model(model, n_classes, is_int=False,
+                         pos_features=False, label_string=False,
+                         random_state=42, is_bool=False,
+                         n_features=20, n_redundant=None,
+                         n_repeated=None):
+    X, y = make_classification(
+        n_classes=n_classes, n_features=n_features, n_samples=250,
+        random_state=random_state, n_informative=min(7, n_features),
+        n_redundant=n_redundant or min(2, n_features - min(7, n_features)),
+        n_repeated=n_repeated or 0)
+    if label_string:
+        y = numpy.array(['cl%d' % cl for cl in y])
+    X = X.astype(numpy.int64) if is_int or is_bool else X.astype(numpy.float32)
+    if pos_features:
+        X = numpy.abs(X)
+    if is_bool:
+        X = X.astype(bool)
+    X_train, X_test = train_test_split(X, test_size=0.5, random_state=42)
+    model.fit(X_train)
+    return model, X_test
+
+
 def fit_multilabel_classification_model(model, n_classes=5, n_labels=2,
                                         n_samples=200, n_features=20,
                                         is_int=False):
@@ -321,19 +343,20 @@ def dump_data_and_model(
     with open(dest, "wb") as f:
         pickle.dump(data, f)
 
-    if hasattr(model, "save"):
-        dest = os.path.join(folder, basename + ".model.keras")
-        names.append(dest)
-        model.save(dest)
-    else:
-        dest = os.path.join(folder, basename + ".model.pkl")
-        names.append(dest)
-        with open(dest, "wb") as f:
-            try:
-                pickle.dump(model, f)
-            except AttributeError as e:
-                print("[dump_data_and_model] cannot pickle model '{}'"
-                      " due to {}.".format(dest, e))
+    dest = os.path.join(folder, basename + ".model.pkl")
+    names.append(dest)
+    load_pickle = True
+    with open(dest, "wb") as f:
+        try:
+            pickle.dump(model, f)
+        except AttributeError as e:
+            print("[dump_data_and_model] cannot pickle model '{}'"
+                  " due to {}.".format(dest, e))
+            load_pickle = False
+    if load_pickle and os.path.exists(dest):
+        # Test unpickle works.
+        with open(dest, "rb") as f:
+            pickle.load(f)
 
     if dump_error_log:
         error_dump = os.path.join(folder, basename + ".err")
@@ -877,6 +900,7 @@ def make_report_backend(folder, as_df=False):
     benched = 0
     files = os.listdir(folder)
     for name in files:
+        print(name)
         if name.endswith(".expected.pkl"):
             model = name.split(".")[0]
             if model not in res:
